/*
** $Id: skycrane.c, initiated January 31, 2013 $
** Skycrane utilities library
** See Copyright Notice in agena.h
*/


#include <stdlib.h>
#include <string.h>
#include <math.h>

#define skycrane_c
#define LUA_LIB

#include "agena.h"

#include "agnxlib.h"
#include "agenalib.h"
#include "agnhlps.h"
#include "agnconf.h"
#include "agncmpt.h"
#include "lobject.h"

#if !(defined(LUA_DOS) || defined(__OS2__) || defined(LUA_ANSI))
#define AGENA_SKYCRANELIBNAME "skycrane"
LUALIB_API int (luaopen_skycrane) (lua_State *L);
#endif


static int skycrane_tocomma (lua_State *L) {
  char *str, *r;
  size_t l, i;
  if (lua_type(L, 1) == LUA_TNUMBER)
    str = (char *)lua_tostring(L, 1);
  else if (lua_type(L, 1) == LUA_TSTRING) {
    str = (char *)agn_tostring(L, 1);
    if (tools_isnumericstring(str) == 0)
      luaL_error(L, "Error in " LUA_QS ": expected a numeric string.", "skycrane.tocomma");
  } else {
    str = NULL;  /* to avoid compiler warning */
    luaL_error(L, "Error in " LUA_QS ": expected a number or numeric string.", "skycrane.tocomma");
  }
  l = strlen(str);
  r = agn_stralloc(L, l, "skycrane.tocomma", NULL);  /* 2.16.5 change */
  for (i=0; i < l; i++) {
    if (*str == '.') r[i] = ','; else r[i] = *str;
    str++;
  }
  r[l] = '\0';
  lua_pushstring(L, r);
  xfree(r);
  return 1;
}


static int skycrane_removedquotes (lua_State *L) {
  size_t l;
  const char *str;
  str = agn_checklstring(L, 1, &l);
  if (l > 1 && str[0] == '"' && str[l - 1] == '"') {
    size_t i;
    char *r;
    r = agn_stralloc(L, l, "skycrane.removedquotes", NULL);  /* 2.16.5 change */
    for (i=0; i < l - 2; i++) r[i] = str[i + 1];
    r[l - 2] = '\0';
    lua_pushstring(L, r);
    xfree(r);
  } else
    lua_pushvalue(L, 1);
  return 1;
}


static int skycrane_enclose (lua_State *L) {
  const char *str, *enc;
  char *r;
  str = lua_tostring(L, 1);
  if (str == NULL)
    luaL_error(L, "Error in " LUA_QS ": expected a string or number for argument #1.", "skycrane.enclose");
  enc = luaL_optstring(L, 2, "\"");
  r = concat(enc, str, enc, NULL);
  if (r == NULL)
    luaL_error(L, "Error in " LUA_QS ": memory allocation failed.", "skycrane.enclose");
  lua_pushstring(L, r);
  xfree(r);
  return 1;
}


static int skycrane_trimpath (lua_State *L) {
  char *str;
  str = (char *)agn_checkstring(L, 1);
  charreplace(str, '\\', '/', 1);
  lua_pushstring(L, str);
  return 1;
}


/* Returns math.branch(ceil(a * log10(x + 1) - 1)), a maximum tolerance value especially suited for
   comparing similar strings and estimating similarities. A good value for a might be a number greater than 3. */
static int skycrane_tolerance (lua_State *L) {  /* 2.10.0, added on September 25, 2016 */
  lua_Number x, a;
  x = agn_checknumber(L, 1);
  a = agn_checknumber(L, 2);
  lua_pushnumber(L, tools_branch(sun_ceil(a*(sun_log10(x + 1) - 1)), 1));
  return 1;
}


static const luaL_Reg skycranelib[] = {
  {"enclose", skycrane_enclose},                /* added on May 26, 2013 */
  {"removedquotes", skycrane_removedquotes},    /* added on February 01, 2013 */
  {"tocomma", skycrane_tocomma},                /* added on January 31, 2013 */
  {"tolerance", skycrane_tolerance},            /* added on September 25, 2016 */
  {"trimpath", skycrane_trimpath},              /* added on February 01, 2013 */
  {NULL, NULL}
};


/*
** Open skycrane library
*/
LUALIB_API int luaopen_skycrane (lua_State *L) {
  luaL_register(L, AGENA_SKYCRANELIBNAME, skycranelib);
  return 1;
}

